@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2012 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: March 2012 */
+/* This file last modified: March 2013 */
 
 
 /* This file contains debugging functions. */
@@ -1106,10 +1106,15 @@ while (s != NULL)
     int flags = s->flags;
     int i, laststave;
 
-    debug_printf("\nSYSTEM\nwarn = %B\nstretch = %B\n"
-      "noadvance = %B\n",
-        flags & sysblock_warn,
-          flags & sysblock_stretch, flags & sysblock_noadvance);
+    debug_printf("\nSYSTEM\n"
+                 "warnkey = %B\n"
+                 "warntime = %B\n" 
+                 "stretch = %B\n"
+                 "noadvance = %B\n",
+        flags & sysblock_warnkey,
+        flags & sysblock_warntime,
+        flags & sysblock_stretch, 
+        flags & sysblock_noadvance);
     debug_printf("bars %d to %d\n", s->barstart, s->barend);
     debug_printf("xjustify = %f\n", s->xjustify);
     debug_printf("barlinewidth = %f\n", s->barlinewidth);
