@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2009 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: April 2009 */
+/* This file last modified: January 2013 */
 
 
 /* This file contains code for writing a MIDI file */
@@ -124,7 +124,7 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
         switch(type)
           {
           case b_Jump:
-          p = (bstr *)(((b_Jumpstr *)p)->next);                    
+          p = (bstr *)(((b_Jumpstr *)p)->next);
           break;
 
           case b_reset:
@@ -202,7 +202,7 @@ for (;;)
     switch(type)
       {
       case b_Jump:
-      p = (bstr *)(((b_Jumpstr *)p)->next);                      
+      p = (bstr *)(((b_Jumpstr *)p)->next);
       break;
 
       case b_nbar:
@@ -365,8 +365,8 @@ if (p != NULL && play_bar >= *p)
     }
   }
 
-/* Now scan the staves. When [notes off] appears in the input, a control is 
-placed at the start of each bar into which it continues, so we do not have to 
+/* Now scan the staves. When [notes off] appears in the input, a control is
+placed at the start of each bar into which it continues, so we do not have to
 keep track between bars. */
 
 for (stave = 1; stave <= play_movt->laststave; stave++)
@@ -382,6 +382,7 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
       int moff = 0;
       int adjustlength = 0;
       int type = p->type;
+      int tremolo = -1;
       BOOL noteson = TRUE;   /* See above comment */
 
       /* Set up midi parameters */
@@ -398,7 +399,7 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
         switch(type)
           {
           case b_Jump:
-          p = (bstr *)(((b_Jumpstr *)p)->next);                      
+          p = (bstr *)(((b_Jumpstr *)p)->next);
           break;
 
           case b_reset:
@@ -496,18 +497,29 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
             }
           break;
 
+          case b_ornament:
+            {
+            b_ornamentstr *orn = (b_ornamentstr *)p;
+            if (orn->ornament == or_trem1 ||
+                orn->ornament == or_trem2)
+              tremolo = orn->ornament;
+            }
+          break;
+
           case b_note:
             {
             b_notestr *note = (b_notestr *)p;
             BOOL thisnotetied = FALSE;
             int length = note->length;
             int nstart = 0;
+            int scrub = 1;
+            int scrubcount;
             int tiebarcount = 1;
             int pitchcount = 0;
             int pitchlist[20];
             int pitchlen[20];
             int pitchstart[20];
-            
+
             oknbar = FALSE;
 
             if (length == 0)
@@ -524,7 +536,7 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
             /* nf_noplay is set when a note has already been played, because of
             a previous tie, which might have been in a previous bar. */
 
-            if ((noteson || midi_for_notes_off) && 
+            if ((noteson || midi_for_notes_off) &&
                 note->spitch != 0 && (note->flags & nf_noplay) == 0)
               {
               /* Get a list of pitches in a chord, and leave the general
@@ -540,21 +552,24 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
                 }
               while (note->type == b_chord);
 
+              /* Advance to start of following note */
+               
+              nstart += length;
+
               /* If the note is followed by a tie, find the next note or
               chord on the stave. If any of its notes have the same pitch as
               any of those in the list, extend their playing times. If there
               are any new notes, add them to the list, with a later starting
               time. We have to do this because all the notes we are
               accumulating will be output at the end of this bar. Set the
-              noplay flag in the next notes, to stop them playing again later.
-              Continue for multiple ties. */
-
-              nstart += length;
+              noplay flag in the next notes, to stop them playing again
+              later. Continue for multiple ties. */
+              
               while (note->type == b_tie)
                 {
                 int i, nlength;
                 note = misc_nextnote(note, NULL);
-                 
+              
                 if (note == NULL &&
                     play_bar + tiebarcount <= play_movt->barcount)
                   {
@@ -564,8 +579,8 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
                     note = misc_nextnote(note, NULL);
                   }
                 if (note == NULL) break;
-                 
-                nlength = note->length; 
+              
+                nlength = note->length;
                 do
                   {
                   for (i = 0; i < pitchcount; i++)
@@ -585,50 +600,72 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
                     note->flags |= nf_noplay;
                     pitchstart[pitchcount++] = nstart;
                     }
-
+              
                   mac_advancechord(note);
                   }
                 while (note->type == b_chord);
                 nstart += nlength;
                 }
 
-              /* For each required pitch, set up the events to make a sound.
-              The lengths and starting times may be different because of
-              tied/non-tied notes in chords. */
-
-              while (--pitchcount >= 0)
+              /* Handle some common scrubbing */
+              
+              if (tremolo > 0 && !thisnotetied)
                 {
-                int pitch = pitchlist[pitchcount] + playtranspose;
-                int start = moff + pitchstart[pitchcount];
-
-                /* We have to schedule a note on and a note off event. Use note
-                on with zero velocity for note off, as that means running
-                status can be used. MIDI middle C is 60; PMW uses 48, so first
-                adjust the pitch. */
-
-                pitch = midi_stave_pitch? midi_stave_pitch : (pitch + 12);
-
-                next_event->time = start;
-                next_event->seq = next_event_seq++;
-                next_event->data[0] = 3;
-                next_event->data[1] = midi_stave_status;
-                next_event->data[2] = pitch;
-                next_event->data[3] = midi_stave_velocity;
-                next_event++;
-
-                next_event->time = start + pitchlen[pitchcount];
-                next_event->seq = next_event_seq++;
-                next_event->data[0] = 3;
-                next_event->data[1] = midi_stave_status;
-                next_event->data[2] = pitch;
-                next_event->data[3] = 0;
-                next_event++;
+                int ttype = (tremolo == or_trem1)? 1 : 2;
+                switch (length)
+                  {
+                  case len_crotchet:       scrub = 2*ttype; break;
+                  case (len_crotchet*3)/2: scrub = 3*ttype; break;
+                  case len_minim:          scrub = 4*ttype; break;
+                  case (len_minim*3)/2:    scrub = 6*ttype; break;
+                  }
                 }
+
+              for (scrubcount = 0; scrubcount < scrub; scrubcount++)
+                { 
+                int pc = pitchcount;
+  
+                /* For each required pitch, set up the events to make a sound.
+                The lengths may be different because of tied/non-tied notes in
+                chords, but these can only happen when not scrubbing. */
+                
+                while (--pc >= 0)
+                  {
+                  int pitch = pitchlist[pc] + playtranspose;
+                  int start = moff + pitchstart[pc] + 
+                    scrubcount * (pitchlen[pc]/scrub);
+                
+                  /* We have to schedule a note on and a note off event. Use
+                  note on with zero velocity for note off, as that means
+                  running status can be used. MIDI middle C is 60; PMW uses 48,
+                  so first adjust the pitch. */
+                
+                  pitch = midi_stave_pitch? midi_stave_pitch : (pitch + 12);
+                
+                  next_event->time = start;
+                  next_event->seq = next_event_seq++;
+                  next_event->data[0] = 3;
+                  next_event->data[1] = midi_stave_status;
+                  next_event->data[2] = pitch;
+                  next_event->data[3] = midi_stave_velocity;
+                  next_event++;
+                
+                  next_event->time = start + (pitchlen[pc]/scrub);
+                  next_event->seq = next_event_seq++;
+                  next_event->data[0] = 3;
+                  next_event->data[1] = midi_stave_status;
+                  next_event->data[2] = pitch;
+                  next_event->data[3] = 0;
+                  next_event++;
+                  }
+                }   
               }
 
             stavetie[stave] = thisnotetied;
             moff += length;
             }
+
+          tremolo = -1;
           break;
           }
 
@@ -861,7 +898,7 @@ if (start_bar > 1)
     }
   }
 
-/* Also, if not starting at the beginning, we must scan through the stave data 
+/* Also, if not starting at the beginning, we must scan through the stave data
 for all preceding bars, in order to pick up any in-line MIDI changes. */
 
 for (play_bar = 1; play_bar < start_bar; play_bar++)
@@ -877,32 +914,32 @@ for (play_bar = 1; play_bar < start_bar; play_bar++)
         switch(type)
           {
           case b_Jump:
-          p = (bstr *)(((b_Jumpstr *)p)->next);                      
+          p = (bstr *)(((b_Jumpstr *)p)->next);
           break;
-  
+
           case b_playchange:
             {
             b_playchangestr *change = (b_playchangestr *)p;
             midi_transpose[stave] += change->transpose;
-  
+
             /* If the relative volume parameter occurs with a change of
             channel, it is a channel volume change. Otherwise it is a
             stave volume change. */
-  
+
             if (change->volume < 128 && change->channel == 128)
               stave_volume[stave] = change->volume;
-  
+
             /* Other changes */
-  
+
             if (change->channel < 128)
               {
               midi_channel[stave] = change->channel;
               if (change->volume < 128)
                 midi_channel_volume[change->channel - 1] = change->volume;
               }
-  
+
             if (change->note < 128) midi_note[stave] = change->note;
-  
+
             if (change->voice < 128)
               {
               writebyte(0);   /* delta time */
@@ -911,17 +948,17 @@ for (play_bar = 1; play_bar < start_bar; play_bar++)
               }
             }
           break;
-  
+
           default:
           break;
           }
-  
+
         p = (bstr *)((uschar *)p + length_table[type]);
         type = p->type;
         }
       }
     }
-  }  
+  }
 
 /* Now write the bars */
 
