@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2012 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: March 2012 */
+/* This file last modified: March 2013 */
 
 
 /* This file contains the main code for creating a position table for a bar. It
@@ -51,7 +51,7 @@ Argument:   TRUE if mis-matched bar lengths in different staves give an error
 Returns:    the width of the bar
 */
 
-int 
+int
 page_makepostable(BOOL lengthwarn)
 {
 posstr *outptr;
@@ -64,7 +64,8 @@ int firstrestbar = TRUE;
 int barnumber = page_barnumber;
 int forcenewline = b_End;
 int endbarmoff = BIGNUMBER;
-int MaxKeyTimeWidth = 0;                   /* widest final key or time */
+int MaxKeyWidth = 0;                       /* widest final key signature */
+int MaxTimeWidth = 0;                      /* widest final time signature */
 int Oldlastendwide = page_lastendwide;     /* previous bar's value */
 int Oldlastenddouble = page_lastenddouble; /* this for double bar */
 int ulaymap[stave_bitvec_size];            /* map underlay staves */
@@ -84,8 +85,8 @@ page_lastendwide = FALSE;         /* flag wide bar line (e.g. repeat) */
 page_lastenddouble = FALSE;       /* this for double barline (not wide) */
 page_manyrest = 0;                /* not many bars rest */
 page_xxwidth = 0;                 /* key and/or time change width */
-page_warnkeytime = FALSE;         /* no warning key or time read */
-
+page_warnkey = FALSE;             /* no warning key read */
+page_warntime = FALSE;            /* no warning time read */
 
 /* Set up initial position table entry for the barline. */
 
@@ -148,7 +149,7 @@ while (barnumber <= curmovt->barcount)
             case b_dbar:
             rrepeatORclefORdbar = TRUE;
             break;
-               
+
             case b_ibar:
             break;
 
@@ -224,7 +225,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
       {
       int type = p->type;
       int moff = 0;                      /* musical offset */
-      int moffdelta;                     /* delta from n*hdsq */ 
+      int moffdelta;                     /* delta from n*hdsq */
       int auxitem = FALSE;               /* last item was aux */
       workposstr *pp = page_postable;    /* position in page_postable */
       mac_setstavesize(page_stave);      /* relative size of this stave */
@@ -255,9 +256,9 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
             int pitch = note->spitch;
             int upflags[stave_bitvec_size];
             int downflags[stave_bitvec_size];
-            
+
             mac_initstave(upflags, 0);
-            mac_initstave(downflags, 0); 
+            mac_initstave(downflags, 0);
 
             if (pitch != 0 || (flags & nf_centre) == 0) notjustrest = TRUE;
 
@@ -304,9 +305,9 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
               barline entry here.) */
 
               if (pp == page_posptr && (page_posptr + 1)->moff == BIGNUMBER)
-                { 
+                {
                 (page_posptr + 1)->xoff = pos_notewidth(length);
-                } 
+                }
 
               /* If we are not at the end of the bar, or if the bar length has
               already been set, set the horizontal offset pro rata pro tem, and
@@ -331,11 +332,11 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
             what is there. */
 
             for (i = 0; i < stave_bitvec_size; i++)
-              { 
+              {
               pp->stemup[i] |= upflags[i];
               pp->stemdown[i] |= downflags[i];
-              } 
-             
+              }
+
             if (extraspace_set)
               {
               if (extraspace >= 0)
@@ -384,9 +385,9 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
           case b_space:
             {
             b_spacestr *bs = (b_spacestr *)p;
-            if (bs->relative) extraspace += (bs->value * main_stavemagn)/1000; 
+            if (bs->relative) extraspace += (bs->value * main_stavemagn)/1000;
               else extraspace += bs->value;
-            }   
+            }
           extraspace_set = TRUE;
           break;
 
@@ -437,7 +438,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
           them when restretching to get even widths. */
 
           case b_name:             /* change of stave name */
-          if (page_layout_stretchn == 1 && page_layout_stretchd == 1) 
+          if (page_layout_stretchn == 1 && page_layout_stretchd == 1)
             {
             int n = ((b_namestr *)p)->n;
             snamestr *s = ((curmovt->stavetable)[page_stave])->stave_name;
@@ -479,18 +480,18 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
       by tuplets. */
 
       if (moff < MaxMoff) moff = MaxMoff;
-       
-      /* If the current length differs from a multiple of hemidemisemiquavers 
-      by only a small amount, round it. This can happen when note lengths are 
-      divided by non-factors for tuplets. For example, in 5/8 time, if one of 
-      the quavers in {5-/6 g-g-g-g-g-g-} is turned into a hemidemisemiquaver, 
+
+      /* If the current length differs from a multiple of hemidemisemiquavers
+      by only a small amount, round it. This can happen when note lengths are
+      divided by non-factors for tuplets. For example, in 5/8 time, if one of
+      the quavers in {5-/6 g-g-g-g-g-g-} is turned into a hemidemisemiquaver,
       the bar ends up off by 1. */
-       
+
       moffdelta = moff % len_hdsquaver;
       if (moffdelta <= TUPLET_ROUND) moff -= moffdelta;
         else if (len_hdsquaver - moffdelta <= TUPLET_ROUND)
-          moff += len_hdsquaver - moffdelta;   
-       
+          moff += len_hdsquaver - moffdelta;
+
       /* Deal with a [space] value */
 
       if (extraspace_set)
@@ -513,9 +514,9 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
 
       if (notjustrest || auxitem)
         {
-        if (endbarmoff == BIGNUMBER) endbarmoff = 0; else 
+        if (endbarmoff == BIGNUMBER) endbarmoff = 0; else
           {
-          if (moff != endbarmoff && lengthwarn) 
+          if (moff != endbarmoff && lengthwarn)
             error_moan(58, barnumber, page_stave, barnumber,
               barnumber, endbarmoff, barnumber, page_stave, moff);
           }
@@ -527,13 +528,13 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
       /* For a whole bar rest, we set the moff in the final entry if it is not
       set (i.e. if this is stave 1), so that subsequent staves space correctly.
       However, whole bar rests are not checked against other bars for length.
-      This makes it easy to handle odd cases by using R!, though it does miss 
+      This makes it easy to handle odd cases by using R!, though it does miss
       the occasional warning that might have been nice. */
 
-      else 
+      else
         {
         if (page_posptr->moff == BIGNUMBER) page_posptr->moff = moff;
-        } 
+        }
       }    /* Block for processing a stave's data */
     }      /* Skip for skipped staves */
   }        /* End of per-stave loop */
@@ -581,12 +582,13 @@ the *right* of each note only -- this includes inverted notes when the stem is
 up. Space to the *left* is dealt with in subsequent passes.
 
 If this is a multiple rest bar we don't need to go through the whole
-rigmarole. */
+rigmarole. The length of such bars is fixed. */
 
 if (page_manyrest >= 2)
   {
+  int i = (curmovt->codemultirests && page_manyrest < 9)? page_manyrest : 0;
+  page_posptr->xoff = longrest_barwidths[i]*main_stavemagn;
   pos_bp->multi = page_manyrest;
-  page_posptr->xoff = 50*main_stavemagn;
   }
 
 else for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
@@ -673,10 +675,10 @@ else for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
             improve on that one day. The wanted width is multiplied by the
             layout stretch factor - this is unity for the first measuring, but
             greater when re-laying-out for wide stretches. */
-            
-            extra = mac_muldiv(wantedwidth, page_layout_stretchn, 
+
+            extra = mac_muldiv(wantedwidth, page_layout_stretchn,
               page_layout_stretchd) - width;
-              
+
             if (extra > 0)
               {
               int x = extra/n;
@@ -786,10 +788,10 @@ below the absolute minimum! */
 for (left = page_postable; left < page_posptr - 1; left++)
   {
   workposstr *right = left + 1;
-  
+
   if (!mac_anystave(left->stemup) && !mac_anystave(right->stemdown) &&
-       mac_anystave(left->stemdown) && 
-       mac_samestave(left->stemdown, right->stemup)) 
+       mac_anystave(left->stemdown) &&
+       mac_samestave(left->stemdown, right->stemup))
     { if (right->xoff >= 11000) right->xoff -= 1000; }
 
   else if (!mac_anystave(left->stemdown) && !mac_anystave(right->stemup) &&
@@ -848,7 +850,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
       int xflags = 0;                     /* flags for encountered items */
       int prevlength = -1;                /* length of previous note */
       int prevflags = 0;                  /* flags on previous note/chord */
-      int ensured = 0; 
+      int ensured = 0;
 
       int gracevector[posx_maxgrace + 1];
       int timevector[posx_maxtime + 1];
@@ -949,13 +951,13 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
               /* Arrange to keep the widest final key/time for warning bars */
               if (timevector[0] > 0)
                 {
-                if (timevector[timevector[0]] > MaxKeyTimeWidth)
-                  MaxKeyTimeWidth = timevector[timevector[0]];
+                if (timevector[timevector[0]] > MaxTimeWidth)
+                  MaxTimeWidth = timevector[timevector[0]];
                 }
-              else if (keyvector[0] > 0)
+              if (keyvector[0] > 0)
                 {
-                if (keyvector[keyvector[0]] > MaxKeyTimeWidth)
-                  MaxKeyTimeWidth = keyvector[keyvector[0]];
+                if (keyvector[keyvector[0]] > MaxKeyWidth)
+                  MaxKeyWidth = keyvector[keyvector[0]];
                 }
 
               /* Now do the insertion work */
@@ -982,20 +984,20 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
 
             /* Handle any ensured value for this note (which previous is now
             pointing at) */
-            
+
             if (ensured > 0)
               {
-              int between = 0; 
+              int between = 0;
               if (prevlength > 0)
                 {
                 workposstr *last = previous - 1;
                 while (last->moff > moff - prevlength)
-                  { between += last->xoff; last--; } 
-                }   
-               
-              if (previous->xoff + between < ensured) 
+                  { between += last->xoff; last--; }
+                }
+
+              if (previous->xoff + between < ensured)
                 { previous->xoff = ensured - between; ensured = 0; }
-              } 
+              }
 
             /* Remember previous note's length and its flags */
 
@@ -1004,10 +1006,10 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
             moff += length;
             }
           break;
-          
+
           case b_ensure:
           ensured = (main_stavemagn * ((b_ensurestr *)p)->value)/1000;
-          break;   
+          break;
 
           /* Deal with non-note items. Clefs, keys, and times at the starts
           of lines will be marked for suppression. */
@@ -1037,7 +1039,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
           case b_dotbar:  xflags |= xf_dotbar;  break;
 
           /* Clefs are the one thing that are allowed at the end of a multiple
-          repeat bar. Fudge the spacing. Update the working copy for use with 
+          repeat bar. Fudge the spacing. Update the working copy for use with
           special key signatures. */
 
           case b_clef:
@@ -1052,7 +1054,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
 
           case b_setclef:
           page_sysclef[page_stave] = ((b_setclefstr *)p)->value;
-          break; 
+          break;
 
           case b_key:
             {
@@ -1066,7 +1068,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
                 misc_keywidth(k->key, page_sysclef[page_stave]);
               keyvector[0] = keycount;
               if (mac_teststave(page_accepteddata->notsuspend, page_stave))
-                page_warnkeytime |= k->warn;
+                page_warnkey |= k->warn;
               }
             }
           break;
@@ -1083,7 +1085,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
               xflags |= xf_keytime;
               if (curmovt->showtime &&
                 mac_teststave(page_accepteddata->notsuspend, page_stave))
-                  page_warnkeytime |= t->warn;
+                  page_warntime |= t->warn;
               }
             }
           break;
@@ -1109,22 +1111,22 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
 
       if (xflags != 0) pos_insertextras(moff, xflags, 0, keyvector, timevector,
         gracevector, previous, prevlength, prevflags);
-        
+
       /* Handle [ensure] at end of bar */
-       
+
       if (ensured > 0)
         {
-        int between = 0; 
+        int between = 0;
         if (prevlength > 0)
           {
           workposstr *last = page_posptr - 1;
           while (last->moff > moff - prevlength)
-            { between += last->xoff; last--; } 
-          }   
+            { between += last->xoff; last--; }
+          }
 
-        if (page_posptr->xoff + between < ensured) 
+        if (page_posptr->xoff + between < ensured)
           page_posptr->xoff = ensured - between;
-        } 
+        }
       }
     }    /* Block for processing a stave's data */
   }      /* End of per-stave loop */
@@ -1179,7 +1181,7 @@ if (curmovt->spreadunderlay)
     if (p != NULL)
       {
       workposstr *previousO = page_postable;
-      workposstr *previousU = page_postable; 
+      workposstr *previousU = page_postable;
       BOOL hadulay = FALSE;
       BOOL hadolay = FALSE;
 
@@ -1238,20 +1240,20 @@ if (curmovt->spreadunderlay)
                 spreadsome = TRUE;
                 }
               }
-              
+
             lastrightU = nextrightU;
             nextleftU = nextrightU = 0;
             hadulay = FALSE;
             previousU = this;
             }
-             
-            
+
+
           /* Similar code for overlay */
 
           if (hadolay && length > 0)
             {
             workposstr *this = previousO;
-            while (this->moff < moff) this++; 
+            while (this->moff < moff) this++;
 
             /* Do nothing at bar start */
 
@@ -1270,7 +1272,7 @@ if (curmovt->spreadunderlay)
                 spreadsome = TRUE;
                 }
               }
-              
+
             lastrightO = nextrightO;
             nextleftO = nextrightO = 0;
             hadolay = FALSE;
@@ -1291,7 +1293,7 @@ if (curmovt->spreadunderlay)
           if ((flags & text_ul) != 0 &&
               (t->ulen != 1 || t->string[0] != '='))
             {
-            int w = 0; 
+            int w = 0;
             int left, right, j;
             int fontsize = mac_muldiv(main_stavemagn,
               ((curmovt->fontsizes)->fontsize_text)[t->size], 1000);
@@ -1301,9 +1303,9 @@ if (curmovt->spreadunderlay)
             uschar ss[256];
             uschar *pp = t->string;
             uschar *qq = ss;
-            uschar *cc = ss; 
-            
-            left = t->x; 
+            uschar *cc = ss;
+
+            left = t->x;
 
             if (matrix != NULL) memcpy(font_transform, matrix, 4*sizeof(int));
 
@@ -1312,35 +1314,35 @@ if (curmovt->spreadunderlay)
             i = 0;
             for (j = 0; j < 2; j++)
               {
-              int k; 
+              int k;
               for (; i < t->ulen && *pp != '^'; i++)
-                { 
+                {
                 if (*pp == '\\')
                   {
-                  int dummy1, dummy2, len; 
+                  int dummy1, dummy2, len;
                   uschar *rr = pp;
-                  uschar dummy3[80]; 
+                  uschar dummy3[80];
                   pp = string_escape(pp+1, dummy3, &dummy1, &dummy2);
                   len = pp - rr;
                   Ustrncpy(qq, rr, len);
-                  qq += len;   
-                  i += len - 1; 
-                  }  
+                  qq += len;
+                  i += len - 1;
+                  }
                 else if (*pp == '#') { *qq++ = ' '; pp++; } else *qq++ = *pp++;
                 }
               *qq = 0;
-              
+
               if (i >= t->ulen || j == 1) break;
-               
+
               for (k = i+1; k < t->ulen; k++) if ((t->string)[k] == '^') break;
-              if (k >= t->ulen) break; 
-              
+              if (k >= t->ulen) break;
+
               left -= string_width(ss, t->font, fontsize);
-              cc = qq; 
+              cc = qq;
               pp++;
-              i++; 
-              } 
-            
+              i++;
+              }
+
             /* Centre if underlay style 0 or if covering one note only or if
             explicitly requested via ^. */
 
@@ -1349,7 +1351,7 @@ if (curmovt->spreadunderlay)
               {
               w = string_width(cc, t->font, fontsize);
               if (w != 0) left += 3*main_stavemagn - w/2;
-              
+
               if (*pp == '^')
                 {
                 same = FALSE;
@@ -1358,32 +1360,32 @@ if (curmovt->spreadunderlay)
                   {
                   if (*pp == '\\')
                     {
-                    int dummy1, dummy2, len; 
+                    int dummy1, dummy2, len;
                     uschar *rr = pp;
-                    uschar dummy3[80]; 
+                    uschar dummy3[80];
                     pp = string_escape(pp+1, dummy3, &dummy1, &dummy2);
                     len = pp - rr;
                     Ustrncpy(qq, rr, len);
-                    qq += len;   
-                    i += len - 1; 
-                    }  
-                  else if (*pp == '#') { *qq++ = ' '; pp++; } 
+                    qq += len;
+                    i += len - 1;
+                    }
+                  else if (*pp == '#') { *qq++ = ' '; pp++; }
                     else *qq++ = *pp++;
                   }
                 *qq = 0;
                 }
               }
-            else same = FALSE;  
-  
+            else same = FALSE;
+
             /* Cut off trailing spaces, but not if they were originally
             # characters. */
 
             pp = t->string + t->ulen;
-            while (qq > ss && qq[-1] == ' ' && pp[-1] != '#') 
-              { 
-              qq--; 
-              pp--; 
-              same = FALSE; 
+            while (qq > ss && qq[-1] == ' ' && pp[-1] != '#')
+              {
+              qq--;
+              pp--;
+              same = FALSE;
               }
             *qq = 0;
 
@@ -1397,7 +1399,7 @@ if (curmovt->spreadunderlay)
 
             if (*ss == ' ')
               {
-              pp = t->string; 
+              pp = t->string;
               qq = ss;
               while (*pp++ != '#' && *qq++ == ' ');  /* Test order matters */
               *qq = 0;
@@ -1422,24 +1424,24 @@ if (curmovt->spreadunderlay)
             font_reset();
             }
           }
-          
+
         /* Deal with resets */
-         
+
         else if (type == b_reset)
           {
           previousO = page_postable;
-          previousU = page_postable; 
+          previousU = page_postable;
           hadulay = FALSE;
           hadolay = FALSE;
-          
+
           nextleftU = 0;
           nextleftO = 0;
           nextrightU = 0;
           nextrightO = 0;
           lastrightU = 0;
           lastrightO = 0;
-          moff = 0; 
-          }   
+          moff = 0;
+          }
 
         /* Move on to next item */
 
@@ -1496,23 +1498,23 @@ if (page_postable->auxid == posx_clef && !page_startlinebar)
     page_postable->xoff -= adjust;
     if (page_postable[1].auxid == 0) page_postable[1].xoff += adjust;
     }
-  
+
   page_postable->xoff += page_startline->clefspace;
 
   while (++t <= page_posptr && posx_keyfirst <= t->auxid && t->auxid <= posx_timelast)
     {
     if (t->auxid <= posx_keylast) t->xoff += page_startline->keyspace;
-      else t->xoff += page_startline->timespace - 4*largestmagn;      
+      else t->xoff += page_startline->timespace - 4*largestmagn;
     }
-     
+
   /***  This would make the spacing as at line start, but stretching makes it
   look bad. If ever the stretching is changed so as not to move the first note
   when it follows other things, instate this, and the same below.
-      
-  if (t <= page_posptr) t->xoff += 3*largestmagn + page_startline->notespace;   
-  ***/ 
+
+  if (t <= page_posptr) t->xoff += 3*largestmagn + page_startline->notespace;
+  ***/
   }
-  
+
 /* If the bar starts with key and/or time signatures, we can reduce the initial
 starting position to be nearer the bar line (provided the barlinewidth is large
 enough), and reduce any gaps between them. However, we must increase the
@@ -1527,44 +1529,65 @@ of lines. At the start of a line, we make the reduction for any signature,
 since the presence of an entry indicates a second signature. */
 
 else if (page_postable->auxid == posx_keyfirst ||
-  page_postable->auxid == posx_timefirst)
+         page_postable->auxid == posx_timefirst)
   {
   workposstr *t = page_postable + (page_startlinebar? 0:1);
-  
+
   page_postable->xoff += (page_postable->auxid == posx_keyfirst)?
-    curmovt->keyspacing : curmovt->timespacing; 
-  
+    curmovt->keyspacing : curmovt->timespacing;
+
   /* Move back start for wide enough barline spacing */
-   
+
   if (page_barlinewidth > 3000)
     page_postable->xoff += 3000 - page_barlinewidth;
-  
-  /* Move forward start if wide bar line */
-      
-  if (Oldlastendwide ||
-    (page_postable->auxid == posx_keyfirst && curmovt->keydoublebar && !Oldlastenddouble))
-      page_postable->xoff += 2000;
 
-  /* Handle multiple signatures & compute special width */
-   
+  /* Move forward start if wide bar line */
+
+  if (Oldlastendwide ||
+      (page_postable->auxid == posx_keyfirst && curmovt->keydoublebar &&
+       !Oldlastenddouble))
+    page_postable->xoff += 2000;
+
+  /* Handle multiple signatures and compute the special width. */
+
   page_xxwidth += page_postable->xoff;
- 
-  while (t <= page_posptr && posx_keyfirst <= t->auxid && t->auxid <= posx_timelast)
+
+  for (; t <= page_posptr && 
+         t->auxid >= posx_keyfirst &&
+         t->auxid <= posx_timelast; t++)
     {
-    if (t->auxid <= posx_keylast) t->xoff -= 3*largestmagn;
-      else t->xoff += page_startline->timespace - 4*largestmagn;      
-    page_xxwidth += (t++)->xoff;
+    if (t->auxid <= posx_keylast)
+      {
+      t->xoff -= 3*largestmagn;
+      if (page_warnkey) page_xxwidth += t->xoff; 
+      }
+    else
+      {
+      t->xoff += page_startline->timespace - 4*largestmagn;
+      if (page_warntime) page_xxwidth += t->xoff; 
+      }
     }
-  page_xxwidth += MaxKeyTimeWidth;  /* for the final item */
-   
+
+  /* Add in space for the final item. MaxTimeWidth has extra space added to it 
+  which seems to be too much in the case when a key signature could have been 
+  present, but was suppressed by [nowarn]. In that case (only), we reduce the 
+  value. This is a fudge because I don't want to mess with the rest of the code 
+  when adding the independent [nowarn] facility. */
+
+  if (!page_warnkey && MaxKeyWidth > 0) MaxTimeWidth -= 8000;
+  
+  page_xxwidth += (page_warnkey && page_warntime)?
+    ((MaxKeyWidth > MaxTimeWidth)? MaxKeyWidth:MaxTimeWidth) :
+    (page_warnkey? MaxKeyWidth : MaxTimeWidth); 
+
   /* Add notespace to midline bars */
 
   /***  This would make the spacing as at line start, but stretching makes it
   look bad. If ever the stretching is changed so as not to move the first note
   when it follows other things, instate this.
-  
-  if (!page_startlinebar && t <= page_posptr) 
-    t->xoff += 3*largestmagn + page_startline->notespace;   
+
+  if (!page_startlinebar && t <= page_posptr)
+    t->xoff += 3*largestmagn + page_startline->notespace;
   ****/
 
   /* If the keys+times are followed by a repeat, bring it nearer too, and if
