@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2010 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: December 2010 */
+/* This file last modified: March 2013 */
 
 
 /* This file contains part II of the code for reading in a PMW score file. It
@@ -140,7 +140,7 @@ if (isdigit(read_ch))
     }
   *((int *)(((uschar *)(curmovt->fontsizes)) + offset)) = read_integer(TRUE);
   *((int **)(((uschar *)(curmovt->fontsizes)) + offset +
-        offsetof(fontsizestr, fontmatrix_music))) = NULL;
+        oo(fontsizestr, fontmatrix_music))) = NULL;
 
   if (read_ch == '/')
     {
@@ -157,7 +157,7 @@ if (isdigit(read_ch))
       {
       int *matrix = store_Xget(4*sizeof(int));
       *((int **)(((uschar *)(curmovt->fontsizes)) + offset +
-        offsetof(fontsizestr, fontmatrix_music))) = matrix;
+        oo(fontsizestr, fontmatrix_music))) = matrix;
       matrix[0] = mac_muldiv(stretch, 65536, 1000);
       matrix[1] = 0;
       matrix[2] = (int)(tan(((double)shear)*atan(1.0)/45000.0)*65536.0);
@@ -720,7 +720,7 @@ clefsize(void)
 sigch();
 if (isdigit(read_ch))
   {
-  set_fontsize(offsetof(fontsizestr,fontsize_clefs), FALSE);
+  set_fontsize(oo(fontsizestr,fontsize_clefs), FALSE);
   (curmovt->fontsizes)->fontsize_clefs *= 10;
   }
 else error_moan(10, "Number");
@@ -1465,20 +1465,20 @@ ptstring(uschar **s, uschar *offset)
 
 if (read_ch == '/')
   {
-  int oo;
+  int fo;
   next_ch();
   if (read_ch == 's')
     {
     next_ch();
-    if (!read_expect_integer(&oo, FALSE, FALSE)) return FALSE;
-    if ((oo -= 1) >= MaxTextFont) error_moan(39, MaxTextFont);
+    if (!read_expect_integer(&fo, FALSE, FALSE)) return FALSE;
+    if ((fo -= 1) >= MaxTextFont) error_moan(39, MaxTextFont);
     }
   else
     {
     error_moan(10, "/s");
     return FALSE;
     }
-  *offset = oo;
+  *offset = fo;
   }
 else *offset = ff_offset_ts;
 
@@ -1885,8 +1885,6 @@ textfont(void)
 {
 int fontid, n;
 uschar s[100];
-uschar s2[100];
-uschar *p;
 
 /* Font selection is permitted only in the first movement */
 
@@ -1904,7 +1902,6 @@ optional second name for the PostScript font. */
 
 if (!read_plainstring(s)) { error_moan(10, "String"); return; }
 if (read_ch == ',') next_ch();
-p = read_plainstring(s2)? s2 : s;
 
 /* See if is already in the font list. If it is, we set its
 number in the font table and return. If not, we set it up
@@ -1943,7 +1940,7 @@ textsizes(void)
 int i;
 for (i = 0; i < MaxTextFont; i++)
   {
-  set_fontsize(offsetof(fontsizestr,fontsize_text) + i*sizeof(int), TRUE);
+  set_fontsize(oo(fontsizestr,fontsize_text) + i*sizeof(int), TRUE);
   if (read_ch == ',') next_ch();
   }
 }
@@ -2070,6 +2067,7 @@ static dirstr read_headlist[] = {
   { "clefsize",         clefsize,       0, 0 },
   { "clefstyle",        clefstyle,      oo(movtstr,clefstyle), int_u },
   { "clefwidths",       clefwidths,     0, 0 },
+  { "codemultirests",   movt_cboolproc, oo(movtstr,codemultirests), TRUE }, 
   { "copyzero",         copyzero,       0, 0 },
   { "cuegracesize",     movt_fszproc,   oo(fontsizestr,fontsize_cuegrace), FALSE },
   { "cuesize",          movt_fszproc,   oo(fontsizestr,fontsize_cue), FALSE },
@@ -2125,6 +2123,7 @@ static dirstr read_headlist[] = {
   { "nobeamendrests",   movt_cboolproc, oo(movtstr,beamendrests), FALSE },
   { "nocheck",          movt_cboolproc, oo(movtstr,check), FALSE },
   { "nocheckdoublebars",movt_cboolproc, oo(movtstr,checkdoublebars), FALSE },
+  { "nocodemultirests", movt_cboolproc, oo(movtstr,codemultirests), FALSE }, 
   { "nokerning",        opt_boolproc,   glob_kerning, FALSE },
   { "nokeywarn",        movt_cboolproc, oo(movtstr,keywarn), FALSE },
   { "nosluroverwarnings", movt_cboolproc, oo(movtstr,tiesoverwarnings), FALSE },
