@@ -5,7 +5,7 @@
 /* Copyright (c) Philip Hazel, 1991 - 2012 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: March 2012 */
+/* This file last modified: January 2013 */
 
 
 /* This file contains part V of the code for reading in a PMW score file - code
@@ -104,7 +104,7 @@ return pitch_stave[pitch] + pitch_clef[stave_clef];
 /* This is also used for dynamics.
 
 Arguments:
-  ax        address of the x movement
+  ax        address of the x movement, or NULL if not allowed
   ay        address of the y movement
   af        address of the bflags field, or NULL if not allowed 
 
@@ -114,7 +114,8 @@ Returns:    nothing
 static void
 ornmove(int *ax, int *ay, uschar *af)
 {
-*ax = *ay = 0;
+*ay = 0;
+if (ax != NULL) *ax = 0;
 if (af != NULL) *af = 0;
 while (read_ch == '/')
   {
@@ -123,8 +124,14 @@ while (read_ch == '/')
     {
     case 'u': *ay += read_movevalue(); break;
     case 'd': *ay -= read_movevalue(); break;
-    case 'l': *ax -= read_movevalue(); break;
-    case 'r': *ax += read_movevalue(); break;
+    
+    case 'l':
+    case 'r':
+    if (ax == NULL)
+      error_moan(10, "/u or /d");
+    else
+      *ax += ((read_ch == 'r')?(+1):(-1)) * read_movevalue();
+    break;          
     
     default:
     if (af == NULL)
@@ -1445,14 +1452,30 @@ TRANSPOSE:
               p->ornament = ap->flag; 
               p->bflags = 0; 
               
-              /* Tremolos have no movement or bracketing; arpeggios and spread
-              have no bracketing, but can have movement. */
+              /* Tremolos have no horizontal movement or bracketing; arpeggios
+              and spread have no bracketing, but can have arbitrary movement. 
+              Because tremelos use '/' as their character, the coding is
+              messy. */
                 
               if (ap->flag >= or_trem1 && ap->flag <= or_trem3)
                 {
-                p->x = p->y = 0;     /* No movement */
+                p->x = 0;            /* No horizontal movement */
+                p->y = 0;            /* Default no vertical movement */
                 p->bflags = 0;       /* No bracketing */
                 ornament = -1;       /* Cancel auto tremolo */
+               
+                if (read_ch == '/')                /* Move a trem3 */
+                  ornmove(NULL, &(p->y), NULL);
+                else if (ap->flag != or_trem1 &&   /* Move a trem1/trem2 */
+                         (read_ch == 'u' || read_ch == 'd') &&
+                         (read_chptr < read_endptr) &&
+                         (isdigit(*read_chptr) || *read_chptr == '-')) 
+                  {
+                  p->ornament -= 1; 
+                  read_ch = '/';
+                  read_chptr--;  
+                  ornmove(NULL, &(p->y), NULL);
+                  } 
                 }  
               else ornmove(&(p->x), &(p->y), 
                 (ap->flag >= or_arp)? NULL : &(p->bflags));
