@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2010 */
+/* Copyright (c) Philip Hazel, 1991 - 2012 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: December 2010 */
+/* This file last modified: March 2012 */
 
 
 /* This file contains code for outputting one bar */
@@ -470,12 +470,13 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
           int ttype = pp->type;
           while (ttype != b_End)
             {
-            if (ttype == b_Jump) p = (bstr *)(((b_Jumpstr *)p)->next);
-              else if (ttype == b_nbar)
-                {
-                flag = TRUE;
-                break;
-                }
+            if (ttype == b_Jump) 
+              pp = (bstr *)(((b_Jumpstr *)pp)->next);
+            else if (ttype == b_nbar)
+              {
+              flag = TRUE;
+              break;
+              }
             pp = (bstr *)((uschar *)pp + length_table[ttype]);
             ttype = pp->type;
             }
