@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2012 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: March 2012 */
+/* This file last modified: March 2013 */
 
 
 /* This file contains code for setting up and outputting beams */
@@ -875,7 +875,7 @@ int type = pp->type;
 int currentmoff = moff;
 
 int longestnote = p->notetype;    /* a secondary beam break forces this */
-int i, xright, yright, pitchleft;
+int i, xright, yright;
 
 BOOL lastopposite;
 BOOL rightbreak2 = FALSE;
@@ -1062,7 +1062,6 @@ beam_firstpitch = p->spitch;
 
 if (beam_upflag && !restatstart) beam_firstX += beam_Xcorrection;
 mac_couplepitch(beam_firstpitch, p->flags);
-pitchleft = beam_firstpitch;
 
 beam_firstY = (beam_firstpitch - 128 + n_upfactor*14)*main_stavemagn;
 
