@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2012 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: March 2012 */
+/* This file last modified: March 2013 */
 
 
 /* This file contains all the major tables used by the program; a few that are
@@ -249,126 +249,175 @@ int stave_tops[] =    { 8000, 8000, 12000, 16000, 12000, 16000, 20000 };
 int stave_bottoms[] = { 8000, 8000,  4000,     0,     0,     0,     0 };
 
 
+/* Data for long rest bars; [0] is for general symbol, [2-8] for special 
+symbols. */
+
+int longrest_barwidths[] = { 50, 0, 30, 35, 30, 35, 35, 45, 40 }; 
+int longrest_widths[]    = { 30000, 0, 4000, 11500, 4000, 11500, 10700, 18200, 
+                             10700 };
+
+
 
 /*************************************************
 *      Structure for printing music chars        *
 *************************************************/
 
-/* This structure converts from an idealized (virtual) "music font" into actual
-characters and positions in the real music font. */
+/* The out_mfstr_ps table below converts from an idealized (virtual) "music
+font" into actual characters and positions in the real music font. The table
+contains pointers to mfstr structures, each of which contains an offset and a
+up to 4 music font characters packed into an unsigned int (i.e. a short
+string), also a chain pointer to another mfstr for further characters at a
+different offset if necessary. */
 
-static mfstr cbaritone= { NULL, 0,  8000, '#' };
-static mfstr tenor    = { NULL, 0,  4000, '#' };
-static mfstr alto     = { NULL, 0,     0, '#' };
-static mfstr mezzo    = { NULL, 0, -4000, '#' };
-static mfstr soprano  = { NULL, 0, -8000, '#' };
+/* C clefs */
 
-static mfstr oldcbaritone= { NULL, 0,  8000, 175 };
-static mfstr oldtenor    = { NULL, 0,  4000, 175 };
-static mfstr oldalto     = { NULL, 0,     0, 175 };
-static mfstr oldmezzo    = { NULL, 0, -4000, 175 };
-static mfstr oldsoprano  = { NULL, 0, -8000, 175 };
+static mfstr alto          = { NULL, 0,     0, '#' };
+static mfstr cbaritone     = { NULL, 0,  8000, '#' };
+static mfstr mezzo         = { NULL, 0, -4000, '#' };
+static mfstr soprano       = { NULL, 0, -8000, '#' };
+static mfstr tenor         = { NULL, 0,  4000, '#' };
 
-static mfstr df1      = { NULL, 3900,  0, '\'' };
-static mfstr dflat    = { &df1, 0,     0, '\'' };
+/* Old-style C clefs */
 
-static mfstr rnatural = { NULL,  0,  0, (142<<16)+('('<<8)+141 };
-static mfstr rsharp   = { NULL,  0,  0, (142<<16)+('%'<<8)+141 };
-static mfstr rdsharp  = { NULL,  0,  0, (142<<16)+('&'<<8)+141 };
+static mfstr oldalto       = { NULL, 0,     0, 175 };
+static mfstr oldcbaritone  = { NULL, 0,  8000, 175 };
+static mfstr oldmezzo      = { NULL, 0, -4000, 175 };
+static mfstr oldsoprano    = { NULL, 0, -8000, 175 };
+static mfstr oldtenor      = { NULL, 0,  4000, 175 };
 
-static mfstr hrsharp1 = { NULL,  0,  0, (142<<16)+(189<<8)+141 };
-static mfstr hrsharp2 = { NULL,  0,  0, (142<<16)+(190<<8)+141 };
+/* Bass clefs, modern and old-style */
 
-static mfstr rf2      = { NULL,  8000,2000,  142 };
-static mfstr rf1      = { &rf2,  3500,   0, '\'' };
-static mfstr rflat    = { &rf1,     0,2000,  141 };
+static mfstr bass          = { NULL, 0,     0, '\"' };
+static mfstr oldbass       = { NULL, 0,     0, 174  };
+static mfstr baritone      = { NULL, 0, -4000, '\"' };
+static mfstr oldbaritone   = { NULL, 0, -4000, 174  };
+static mfstr deepbass      = { NULL, 0,  4000, '\"' };
+static mfstr olddeepbass   = { NULL, 0,  4000, 174  };
 
-static mfstr hrf12    = { NULL,  8000,2000,  142 };
-static mfstr hrf11    = { &hrf12, 3500,  0,  191 };
-static mfstr hrflat1  = { &hrf11,   0,2000,  141 };
+static mfstr cbp1          = { NULL, 2500, -6000, ']'  };
+static mfstr contrabass    = { &cbp1,   0,     0, '\"' };
+static mfstr oldcontrabass = { &cbp1,   0,     0, 174  };
 
-static mfstr hrf22    = { NULL,  8000,2000,  142 };
-static mfstr hrf21    = { &hrf22,  3500, 0,  192 };
-static mfstr hrflat2  = { &hrf21,   0,2000,  141 };
+static mfstr sbp1          = { NULL, 3500, 16500, ']'  };
+static mfstr soprabass     = { &sbp1,   0,     0, '\"' };
+static mfstr oldsoprabass  = { &sbp1,   0,     0, 174  };
 
-static mfstr rdf3     = { NULL, 11900,2000,  142 };
-static mfstr rdf2     = { &rdf3, 7400,   0, '\'' };
-static mfstr rdf1     = { &rdf2, 3500,   0, '\'' };
-static mfstr rdflat   = { &rdf1,    0,2000,  141 };
+/* Treble clefs */
 
-static mfstr snatural = { NULL,  0,  0, (140<<16)+('('<<8)+139 };
-static mfstr ssharp   = { NULL,  0,  0, (140<<16)+('%'<<8)+139 };
-static mfstr sdsharp  = { NULL,  0,  0, (140<<16)+('&'<<8)+139 };
+static mfstr treble        = { NULL, 0, 0, '!' };
 
-static mfstr hssharp1 = { NULL,  0,  0, (140<<16)+(189<<8)+139 };
-static mfstr hssharp2 = { NULL,  0,  0, (140<<16)+(190<<8)+139 };
+static mfstr ttp1          = { NULL, 4000, -12000, ']' };
+static mfstr trebletenor   = { &ttp1,   0,      0, '!' };
 
-static mfstr sf2      = { NULL,  8000,2000,  140 };
-static mfstr sf1      = { &sf2,  3500,   0, '\'' };
-static mfstr sflat    = { &sf1,     0,2000,  139 };
+static mfstr ttp4          = { NULL,  9300, -12000, 158 };
+static mfstr ttp3          = { &ttp4, 1000, -12000, 157 };
+static mfstr ttp2          = { &ttp3, 4300, -12000, ']' };
+static mfstr trebletenorB  = { &ttp2,    0,      0, '!' };
 
-static mfstr hsf12    = { NULL,  8000,2000,  140 };
-static mfstr hsf11    = { &hsf12, 3500,   0, 191 };
-static mfstr hsflat1  = { &hsf11,   0,2000,  139 };
+static mfstr tdp1          = { NULL, 6000, 23000, ']' };
+static mfstr trebledescant = { &tdp1,   0,     0, '!' };
 
-static mfstr hsf22    = { NULL,  8000,2000,  140 };
-static mfstr hsf21    = { &hsf22, 3500,   0, 192 };
-static mfstr hsflat2  = { &hsf21,   0,2000,  139 };
+/* Accidentals */
 
-static mfstr sdf3     = { NULL, 11900,2000,  140 };
-static mfstr sdf2     = { &sdf3, 7400,   0, '\'' };
-static mfstr sdf1     = { &sdf2, 3500,   0, '\'' };
-static mfstr sdflat   = { &sdf1,    0,2000,  139 };
+static mfstr natural       = { NULL, 0, 0, '(' };
+static mfstr sharp         = { NULL, 0, 0, '%' };
+static mfstr flat          = { NULL, 0, 0, '\'' };
+static mfstr doublesharp   = { NULL, 0, 0, '&' };
 
-static mfstr ttp1 = { NULL, 4000, -12000, ']' };
-static mfstr trebletenor = { &ttp1, 0, 0, '!' };
+static mfstr df1           = { NULL, 3900,  0, '\'' };
+static mfstr doubleflat    = { &df1, 0,     0, '\'' };
 
-static mfstr ttp4 = { NULL,  9300, -12000, 158 };
-static mfstr ttp3 = { &ttp4, 1000, -12000, 157 };
-static mfstr ttp2 = { &ttp3, 4300, -12000, ']' };
-static mfstr trebletenorB = { &ttp2, 0, 0, '!' };
+/* Round-bracketed accidentals */
+                           
+static mfstr rnatural      = { NULL,  0,  0, (142<<16)+('('<<8)+141 };
+static mfstr rsharp        = { NULL,  0,  0, (142<<16)+('%'<<8)+141 };
+static mfstr rdsharp       = { NULL,  0,  0, (142<<16)+('&'<<8)+141 };
+                           
+static mfstr rf2           = { NULL,  8000,2000,  142 };
+static mfstr rf1           = { &rf2,  3500,   0, '\'' };
+static mfstr rflat         = { &rf1,     0,2000,  141 };
+                           
+static mfstr rdf3          = { NULL, 11900,2000,  142 };
+static mfstr rdf2          = { &rdf3, 7400,   0, '\'' };
+static mfstr rdf1          = { &rdf2, 3500,   0, '\'' };
+static mfstr rdflat        = { &rdf1,    0,2000,  141 };
 
-static mfstr tdp1 = { NULL, 6000, 23000, ']' };
-static mfstr trebledescant = { &tdp1, 0, 0, '!' };
+/* Square-bracket accidentals */
+                           
+static mfstr snatural      = { NULL,  0,  0, (140<<16)+('('<<8)+139 };
+static mfstr ssharp        = { NULL,  0,  0, (140<<16)+('%'<<8)+139 };
+static mfstr sdsharp       = { NULL,  0,  0, (140<<16)+('&'<<8)+139 };
+                           
+static mfstr sf2           = { NULL,  8000,2000,  140 };
+static mfstr sf1           = { &sf2,  3500,   0, '\'' };
+static mfstr sflat         = { &sf1,     0,2000,  139 };
+                           
+static mfstr sdf3          = { NULL, 11900,2000,  140 };
+static mfstr sdf2          = { &sdf3, 7400,   0, '\'' };
+static mfstr sdf1          = { &sdf2, 3500,   0, '\'' };
+static mfstr sdflat        = { &sdf1,    0,2000,  139 };
 
-static mfstr cbp1 = { NULL, 2500, -6000, ']' };
-static mfstr contrabass = { &cbp1, 0, 0, '\"' };
-static mfstr oldcontrabass = { &cbp1, 0, 0, 174 };
+/* Half accidentals */
 
-static mfstr sbp1 = { NULL, 3500, 16500, ']' };
-static mfstr soprabass = { &sbp1, 0, 0, '\"' };
-static mfstr oldsoprabass = { &sbp1, 0, 0, 174 };
+static mfstr halfsharp1    = { NULL, 0, 0, 189 };
+static mfstr halfsharp2    = { NULL, 0, 0, 190 };
+static mfstr halfflat1     = { NULL, 0, 0, 191 };
+static mfstr halfflat2     = { NULL, 0, 0, 192 };
 
-static mfstr baritone = { NULL, 0, -4000, '\"' };
-static mfstr deepbass = { NULL, 0,  4000, '\"' };
+/* Round-bracketed half accidentals */
 
-static mfstr oldbaritone = { NULL, 0, -4000, 174 };
-static mfstr olddeepbass = { NULL, 0,  4000, 174 };
+static mfstr hrsharp1      = { NULL,  0,  0, (142<<16)+(189<<8)+141 };
+static mfstr hrsharp2      = { NULL,  0,  0, (142<<16)+(190<<8)+141 };
+                           
+static mfstr hrf12         = { NULL,   8000, 2000,  142 };
+static mfstr hrf11         = { &hrf12, 3500,    0,  191 };
+static mfstr hrflat1       = { &hrf11,    0, 2000,  141 };
+                           
+static mfstr hrf22         = { NULL,   8000, 2000,  142 };
+static mfstr hrf21         = { &hrf22, 3500,    0,  192 };
+static mfstr hrflat2       = { &hrf21,    0, 2000,  141 };
 
-/* Small integers in these tables indicate a one-to-one correspondence between
-the musical character and a character in the music font. */
+/* Square-bracketed half-accidentals */
+                           
+static mfstr hssharp1      = { NULL,  0,  0, (140<<16)+(189<<8)+139 };
+static mfstr hssharp2      = { NULL,  0,  0, (140<<16)+(190<<8)+139 };
+
+static mfstr hsf12         = { NULL,   8000, 2000, 140 };
+static mfstr hsf11         = { &hsf12, 3500,    0, 191 };
+static mfstr hsflat1       = { &hsf11,    0, 2000, 139 };
+                           
+static mfstr hsf22         = { NULL,   8000, 2000, 140 };
+static mfstr hsf21         = { &hsf22, 3500,    0, 192 };
+static mfstr hsflat2       = { &hsf21,    0, 2000, 139 };
+                           
+/* Miscllaneous */
+
+static mfstr common        = { NULL, 0, 0, '^' };
+static mfstr cut           = { NULL, 0, 0, '_' };
+static mfstr hclef         = { NULL, 0, 0, 173 };
+static mfstr longrest      = { NULL, 0, 0, '0' };
+
+
+/* This table is indexed by "virtual music font" character. These are defined 
+as mc_xxx enums in outhdr.h and must be kept in step with this table. The order 
+is arbitrary. */
 
 mfstr *out_mftable_ps[] =
   {
-  (mfstr *)33, &trebletenor, &trebledescant,
-  (mfstr *)34, &contrabass,  &soprabass,
-  &cbaritone,  &tenor,       &alto, 
-  &mezzo,      &soprano,
-
-  (mfstr *)40, (mfstr *)37,    (mfstr *)38, (mfstr *)39, &dflat,
-  &rnatural,   &rsharp,        &rdsharp,    &rflat,      &rdflat,
-  &snatural,   &ssharp,        &sdsharp,    &sflat,      &sdflat,
-
-  (mfstr *)94, (mfstr *)95,                            /* common & cut */
-  (mfstr *)48, &trebletenorB, (mfstr *)173,            /* long rest & clefs */
-  &baritone,   &deepbass,     (mfstr *)174, &oldcontrabass,
-  &oldsoprabass, &oldcbaritone, &oldtenor,  &oldalto,  &oldmezzo,   
-  &oldsoprano, &oldbaritone, &olddeepbass,
-  
-  (mfstr *)189, &hrsharp1, &hssharp1, 
-  (mfstr *)190, &hrsharp2, &hssharp2, 
-  (mfstr *)191, &hrflat1, &hsflat1, 
-  (mfstr *)192, &hrflat2, &hsflat2
+  &treble,       &trebletenor,  &trebledescant, &bass,      
+  &contrabass,   &soprabass,    &cbaritone,     &tenor,       
+  &alto,         &mezzo,        &soprano,       &natural,  
+  &sharp,        &doublesharp,  &flat,          &doubleflat,
+  &rnatural,     &rsharp,       &rdsharp,       &rflat, 
+  &rdflat,       &snatural,     &ssharp,        &sdsharp,     
+  &sflat,        &sdflat,       &common,        &cut,
+  &longrest,     &trebletenorB, &hclef,         &baritone,   
+  &deepbass,     &oldbass,      &oldcontrabass, &oldsoprabass, 
+  &oldcbaritone, &oldtenor,     &oldalto,       &oldmezzo,   
+  &oldsoprano,   &oldbaritone,  &olddeepbass,   &halfsharp1, 
+  &hrsharp1,     &hssharp1,     &halfsharp2,    &hrsharp2, 
+  &hssharp2,     &halfflat1,    &hrflat1,       &hsflat1, 
+  &halfflat2,    &hrflat2,      &hsflat2
   };
 
 
@@ -678,6 +727,7 @@ movtstr init_movtstr = {
   FALSE,              /* breverests */
   TRUE,               /* check */
   TRUE,               /* checkdoublebars */
+  FALSE,              /* codemultirests */ 
   TRUE,               /* fullbarend */
   FALSE,              /* incipitjoin */
   TRUE,               /* keydoublebar */
